@@ -1,7 +1,7 @@
 #ifndef HSSYSTEMFILEIO_UNIX_H
 #define HSSYSTEMFILEIO_UNIX_H
 
-#include <dirent.h>
+struct dirent;
 
 struct dirent *
 hssystemfileio_alloc_dirent();
@@ -10,7 +10,7 @@ void
 hssystemfileio_free_dirent(struct dirent *);
 
 int
-hssystemfileio_readdir(DIR *dir, struct dirent *dirent);
+hssystemfileio_readdir(void *dir, struct dirent *dirent);
 
 char *
 hssystemfileio_dirent_name(struct dirent *dirent);
